#include <zephyr.h>
#include <cJSON.h>
#include "cJSON_os.h"
static int json_add_obj(cJSON *parent, const char *str, cJSON *item)
{
	cJSON_AddItemToObject(parent, str, item);

	return 0;
}

static int json_add_number(cJSON *parent, const char *str, double item)
{
	cJSON *json_num;

	json_num = cJSON_CreateNumber(item);
	if (json_num == NULL) {
		return -ENOMEM;
	}

	return json_add_obj(parent, str, json_num);
}

struct data_time{
    uint64_t time1;
};

K_MSGQ_DEFINE(data_timeq, sizeof(struct data_time), 1, 1);
K_MSGQ_DEFINE(data_dataq, sizeof(uint8_t), 1, 1);
// This is how I call the time_q in data_gen.c file to withe a variable in the queue:

#if 0
int data_gen(){
    uint64_t message_ts = 0;

    int err = date_time_now(&message_ts);
    if (err) {
        printk("date_time_now, error: %d\n", err);
        return err;
    }
    aws_put_to_dataq_timeq(1, message_ts);
}
#endif

struct k_msgq* time_q1;
struct k_msgq* data_q1;
struct k_msgq* time_q;
struct k_msgq* data_q;


void register_msgq_addres1(struct k_msgq* ptr_from_main_data, struct k_msgq* ptr_from_main_time){
    data_q = ptr_from_main_data;
    time_q = ptr_from_main_time;
}

void aws_put_to_dataq_timeq(uint8_t put_data, uint64_t put_time){
   
    if (data_q == NULL){
        printk("Eda msgq is null! Did you forget to call register_msgq_address?");
        return;
    }
    if (time_q == NULL){
        printk("Eda msgq is null! Did you forget to call register_msgq_address?");
        return;
    }
   
    int err = k_msgq_put(data_q, &put_data, K_NO_WAIT);
    if (err) {
        printk("Failed to put data in data_q: %d\n", err);
    }
    err = k_msgq_put(time_q, &put_time, K_NO_WAIT);
    if (err) {
        printk("Failed to put data in time_q: %d\n", err);
    }
 
}

void producer_thread(void) {
    printk("Producer thread starting\n");
	uint64_t time = 8589934000;
    uint8_t data = 0;

	while (1) {
        data++;
        printk("Putting data in queue\n");
        aws_put_to_dataq_timeq(data, time + data);

		k_sleep(K_MSEC(500));
	}
}

void aws_get_from_q(uint64_t* get_time, uint8_t *get_data){
        k_msgq_get(time_q1, get_time, K_NO_WAIT);
        k_msgq_get(data_q1, get_data, K_NO_WAIT);
}

//and this is in MQTT_AWS.c file how I read the data from the queue:
static int data_publish()
{
    int err=0;
    char *message;
    uint64_t message_ts = 0;
    uint8_t sen_val = 0;
    cJSON *root_obj = NULL;
 

    if(k_msgq_num_used_get(data_q1) > 0){
        if(k_msgq_num_used_get(time_q1) > 0){
            aws_get_from_q(&message_ts, &sen_val);
            printk("Got message\nts: %llu\nval: %d\n", message_ts, sen_val);
            if (root_obj == NULL){
                printk("Root obj is NULL\n");
                cJSON_Delete(root_obj);
                err = -ENOMEM;
                return err;
            }

            err += json_add_number(root_obj, "sen", sen_val);
            err += json_add_number(root_obj, "ts", message_ts);

            if (err) {
                printk("json_add, error: %d\n", err);
                goto cleanup;
            }

            message = cJSON_Print(root_obj);
            if (message == NULL) {
                printk("cJSON_Print, error: returned NULL\n");
                err = -ENOMEM;
                goto cleanup;
            }

            printk("Publishing: %s to AWS IoT broker\n", message);

            //err = aws_iot_send(&tx_data);
            if (err) {
                printk("aws_iot_send, error: %d\n", err);
            }

            cJSON_FreeString(message);
           
        }
       
    } else {
        printk("No data in dataq\n");
    }
   
cleanup:
    cJSON_Delete(root_obj);
   
    return err;
}

void register_msgq_addres2(struct k_msgq* ptr_from_main_data, struct k_msgq* ptr_from_main_time){
    data_q1 = ptr_from_main_data;
    time_q1 = ptr_from_main_time;
}

K_THREAD_DEFINE(my_tid, 1024, producer_thread, NULL, NULL, NULL, CONFIG_MAIN_THREAD_PRIORITY, 0, 10);

void main(void) 
{
    register_msgq_addres1(&data_dataq, &data_timeq);
    register_msgq_addres2(&data_dataq, &data_timeq);

    printk("&data_dataq:\t%x\n", &data_dataq);
    printk("data_q:\t\t%x\n", data_q);
    printk("data_q1:\t\t%x\n", data_q1);
    
    while (1) {
        printk("Getting msg from queue\n");
        data_publish();
        k_sleep(K_MSEC(200));
    }

}